/*
 *  arch/arm/mach-vt8500/bv07.c
 *
 *  Copyright (C) 2010 Alexey Charkov <alchark@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <linux/io.h>

#include <mach/vt8500.h>
#include "devices.h"

static struct platform_device *devices[] __initdata = {
	&vt8500_device_uart0,
#ifdef CONFIG_FB_VT8500
	&vt8500_device_lcdc,
#endif
#ifdef CONFIG_USB_EHCI_HCD
	&vt8500_device_ehci,
#endif
#ifdef CONFIG_FB_WMT_GE_ROPS
	&vt8500_device_ge_rops,
#endif
#ifdef CONFIG_HAVE_PWM
	&vt8500_device_pwm0,
#endif
#ifdef CONFIG_BACKLIGHT_PWM
	&vt8500_device_pwmbl,
#endif
#ifdef CONFIG_RTC_DRV_VT8500
	&vt8500_device_rtc,
#endif
};

void __init bv07_init(void)
{
#ifdef CONFIG_FB_VT8500
	void __iomem *gpio_mux_reg = ioremap(VT8500_GPIO_BASE + 0x200, 4);

	writel(readl(gpio_mux_reg) | 1, gpio_mux_reg);
	iounmap(gpio_mux_reg);
#endif
	platform_add_devices(devices, ARRAY_SIZE(devices));
	vt8500_gpio_init();
}

MACHINE_START(BV07, "Benign BV07 Mini Netbook")
	.phys_io	= VT8500_REGS_START_PHYS,
	.io_pg_offst	= ((VT8500_REGS_START_VIRT) >> 18) & 0xfffc,
	.boot_params	= 0x00000100,
	.map_io		= vt8500_map_io,
	.init_irq	= vt8500_init_irq,
	.timer		= &vt8500_timer,
	.init_machine	= bv07_init,
MACHINE_END
