/* linux/arch/arm/mach-vt8500/devices.c
 *
 * Copyright (C) 2010 Alexey Charkov <alchark@gmail.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/io.h>
#include <linux/device.h>
#include <linux/dma-mapping.h>
#include <linux/platform_device.h>
#include <linux/pwm_backlight.h>
#include <linux/bootmem.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>

#include <mach/irqs.h>
#include <mach/hardware.h>
#include <mach/vt8500.h>
#include "devices.h"

#ifdef CONFIG_FB_VT8500
static struct resource resources_lcdc[] = {
	{
		.start	= VT8500_LCDC_BASE,
		.end	= VT8500_LCDC_BASE + SZ_1K - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= IRQ_LCDC,
		.end	= IRQ_LCDC,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= 0,
		.end	= 0,
		.flags	= IORESOURCE_MEM,
	}
};

static u64 fb_dma_mask = DMA_BIT_MASK(32);

struct platform_device vt8500_device_lcdc = {
	.name           = "vt8500-lcd",
	.id             = 0,
	.dev		= {
		.dma_mask	= &fb_dma_mask,
		.coherent_dma_mask = DMA_BIT_MASK(32),
	},
	.num_resources  = ARRAY_SIZE(resources_lcdc),
	.resource       = resources_lcdc,
};
#endif

#ifdef CONFIG_FB_WM8505
static struct resource resources_wm8505_fb[] = {
	{
		.start	= WM8505_GOVR_BASE,
		.end	= WM8505_GOVR_BASE + 512 - 1,
		.flags	= IORESOURCE_MEM,
	}
};

struct platform_device vt8500_device_wm8505_fb = {
	.name           = "wm8505-fb",
	.id             = 0,
	.num_resources  = ARRAY_SIZE(resources_wm8505_fb),
	.resource       = resources_wm8505_fb,
};
#endif

static struct map_desc vt8500_io_desc[] __initdata = {
	{
		.virtual	=  VT8500_REGS_START_VIRT,
		.pfn		= __phys_to_pfn(VT8500_REGS_START_PHYS),
		.length		= VT8500_REGS_LENGTH,
		.type		= MT_DEVICE
	},
};

void __init vt8500_map_io(void)
{
#ifdef CONFIG_FB_VT8500
	resources_lcdc[2].start = (unsigned long)__alloc_bootmem(SZ_4M, SZ_4M, 0);
	resources_lcdc[2].end = resources_lcdc[2].start + SZ_4M - 1;
#endif
	iotable_init(vt8500_io_desc, ARRAY_SIZE(vt8500_io_desc));
}

static struct resource resources_uart0[] = {
	{
		.start	= IRQ_UART0,
		.end	= IRQ_UART0,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= VT8500_UART0_BASE,
		.end	= VT8500_UART0_BASE + VT8500_UART_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct resource resources_uart1[] = {
	{
		.start	= IRQ_UART1,
		.end	= IRQ_UART1,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= VT8500_UART1_BASE,
		.end	= VT8500_UART1_BASE + VT8500_UART_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct resource resources_uart2[] = {
	{
		.start	= IRQ_UART2,
		.end	= IRQ_UART2,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= VT8500_UART2_BASE,
		.end	= VT8500_UART2_BASE + VT8500_UART_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct resource resources_uart3[] = {
	{
		.start	= IRQ_UART3,
		.end	= IRQ_UART3,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= VT8500_UART3_BASE,
		.end	= VT8500_UART3_BASE + VT8500_UART_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

#ifdef CONFIG_VTWM_VERSION_WM8505
static struct resource resources_uart4[] = {
	{
		.start	= IRQ_UART4,
		.end	= IRQ_UART4,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= VT8500_UART4_BASE,
		.end	= VT8500_UART4_BASE + VT8500_UART_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct resource resources_uart5[] = {
	{
		.start	= IRQ_UART5,
		.end	= IRQ_UART5,
		.flags	= IORESOURCE_IRQ,
	},
	{
		.start	= VT8500_UART5_BASE,
		.end	= VT8500_UART5_BASE + VT8500_UART_SIZE - 1,
		.flags	= IORESOURCE_MEM,
	},
};
#endif

struct platform_device vt8500_device_uart0 = {
	.name		= "vt8500_serial",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(resources_uart0),
	.resource	= resources_uart0,
};

struct platform_device vt8500_device_uart1 = {
	.name		= "vt8500_serial",
	.id		= 1,
	.num_resources	= ARRAY_SIZE(resources_uart1),
	.resource	= resources_uart1,
};

struct platform_device vt8500_device_uart2 = {
	.name		= "vt8500_serial",
	.id		= 2,
	.num_resources	= ARRAY_SIZE(resources_uart2),
	.resource	= resources_uart2,
};

struct platform_device vt8500_device_uart3 = {
	.name		= "vt8500_serial",
	.id		= 3,
	.num_resources	= ARRAY_SIZE(resources_uart3),
	.resource	= resources_uart3,
};

#ifdef CONFIG_VTWM_VERSION_WM8505
struct platform_device vt8500_device_uart4 = {
	.name		= "vt8500_serial",
	.id		= 4,
	.num_resources	= ARRAY_SIZE(resources_uart4),
	.resource	= resources_uart4,
};

struct platform_device vt8500_device_uart5 = {
	.name		= "vt8500_serial",
	.id		= 5,
	.num_resources	= ARRAY_SIZE(resources_uart5),
	.resource	= resources_uart5,
};
#endif

#ifdef CONFIG_USB_EHCI_HCD
static struct resource resources_ehci[] = {
	{
		.start	= VT8500_EHCI_BASE,
		.end	= VT8500_EHCI_BASE + 512 - 1,
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= IRQ_EHCI,
		.end	= IRQ_EHCI,
		.flags	= IORESOURCE_IRQ,
	}
};

static u64 ehci_dma_mask = DMA_BIT_MASK(32);

struct platform_device vt8500_device_ehci = {
	.name		= "vt8500-ehci",
	.id		= 0,
	.dev		= {
		.dma_mask	= &ehci_dma_mask,
		.coherent_dma_mask = DMA_BIT_MASK(32),
	},
	.num_resources	= ARRAY_SIZE(resources_ehci),
	.resource	= resources_ehci,
};
#endif

#ifdef CONFIG_FB_WMT_GE_ROPS
static struct resource resources_ge_rops[] = {
	{
		.start	= VT8500_GEGEA_BASE,
		.end	= VT8500_GEGEA_BASE + 0xff,
		.flags	= IORESOURCE_MEM,
	}
};

struct platform_device vt8500_device_ge_rops = {
	.name		= "wmt_ge_rops",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(resources_ge_rops),
	.resource	= resources_ge_rops,
};
#endif

#ifdef CONFIG_HAVE_PWM
static struct resource resources_pwm0[] = {
	[0] = {
		.start	= VT8500_PWM_BASE,
		.end	= VT8500_PWM_BASE + 0xf,
		.flags	= IORESOURCE_MEM,
	},
};

struct platform_device vt8500_device_pwm0 = {
	.name		= "vt8500-pwm",
	.id		= 0,
	.resource	= resources_pwm0,
	.num_resources	= ARRAY_SIZE(resources_pwm0),
};
#endif

#ifdef CONFIG_BACKLIGHT_PWM
static struct platform_pwm_backlight_data vt8500_pwmbl_data = {
	.pwm_id		= 0,
	.max_brightness	= 128,
	.dft_brightness = 70,
	.pwm_period_ns	= 250000, /* revisit when clocks are implemented */
};

struct platform_device vt8500_device_pwmbl = {
	.name		= "pwm-backlight",
	.id		= 0,
	.dev		= {
		.platform_data = &vt8500_pwmbl_data,
	},
};
#endif

#ifdef CONFIG_RTC_DRV_VT8500
static struct resource resources_rtc[] = {
	[0] = {
		.start	= VT8500_RTC_BASE,
		.end	= VT8500_RTC_BASE + 0x2c - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= IRQ_RTC,
		.end	= IRQ_RTC,
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		.start	= IRQ_RTCSM,
		.end	= IRQ_RTCSM,
		.flags	= IORESOURCE_IRQ,
	},
};

struct platform_device vt8500_device_rtc = {
	.name		= "vt8500-rtc",
	.id		= 0,
	.resource	= resources_rtc,
	.num_resources	= ARRAY_SIZE(resources_rtc),
};
#endif
