#ifndef _I8042_VT8500_H
#define _I8042_VT8500_H

#include <mach/irqs.h>
#include <mach/vt8500.h>

/*
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 */

static void __iomem *regbase;

/*
 * Names.
 */

#define I8042_KBD_PHYS_DESC "vt8500ps2/serio0"
#define I8042_AUX_PHYS_DESC "vt8500ps2/serio1"
#define I8042_MUX_PHYS_DESC "vt8500ps2/serio%d"

/*
 * IRQs.
 */

#define I8042_KBD_IRQ	IRQ_PS2KBD
#define I8042_AUX_IRQ	IRQ_PS2MOUSE


/*
 * Register numbers.
 */

#define I8042_COMMAND_REG	(regbase + 0x4)
#define I8042_STATUS_REG	(regbase + 0x4)
#define I8042_DATA_REG		(regbase + 0x0)

static inline int i8042_read_data(void)
{
	return readl(I8042_DATA_REG);
}

static inline int i8042_read_status(void)
{
	return readl(I8042_STATUS_REG);
}

static inline void i8042_write_data(int val)
{
	writel(val, I8042_DATA_REG);
}

static inline void i8042_write_command(int val)
{
	writel(val, I8042_COMMAND_REG);
}

static inline int i8042_platform_init(void)
{
	i8042_reset = 1;
	regbase = ioremap(VT8500_PS2_BASE, SZ_1K);
	return 0;
}

static inline void i8042_platform_exit(void)
{
	iounmap(regbase);
}

#endif /* _I8042_VT8500_H */
